@@ -31,20 +31,26 @@ import sys
 import os
 import signal
 import struct
-import cStringIO as StringIO
+from io import BytesIO
 import select
 import socket
 import errno
 import traceback
 
 try:
-    import thread
-    import threading
-    thread_available = True
+    import _thread as thread  # py3
 except ImportError:
-    import dummy_thread as thread
-    import dummy_threading as threading
-    thread_available = False
+    import thread  # py2
+
+import threading
+thread_available = True
+
+if sys.hexversion >= 0x03000000:
+    text_type = str
+    force_str = lambda s: s.decode('latin-1')
+else:
+    text_type = unicode
+    force_str = lambda s: s
 
 # Apparently 2.3 doesn't define SHUT_WR? Assume it is 1 in this case.
 if not hasattr(socket, 'SHUT_WR'):
@@ -126,7 +132,7 @@ class InputStream(object):
         # See Server.
         self._shrinkThreshold = conn.server.inputStreamShrinkThreshold
 
-        self._buf = ''
+        self._buf = b''
         self._bufList = []
         self._pos = 0 # Current read position.
         self._avail = 0 # Number of bytes currently available.
@@ -148,7 +154,7 @@ class InputStream(object):
 
     def read(self, n=-1):
         if self._pos == self._avail and self._eof:
-            return ''
+            return b''
         while True:
             if n < 0 or (self._avail - self._pos) < n:
                 # Not enough data available.
@@ -165,7 +171,7 @@ class InputStream(object):
                 break
         # Merge buffer list, if necessary.
         if self._bufList:
-            self._buf += ''.join(self._bufList)
+            self._buf += b''.join(self._bufList)
             self._bufList = []
         r = self._buf[self._pos:newPos]
         self._pos = newPos
@@ -174,14 +180,14 @@ class InputStream(object):
 
     def readline(self, length=None):
         if self._pos == self._avail and self._eof:
-            return ''
+            return b''
         while True:
             # Unfortunately, we need to merge the buffer list early.
             if self._bufList:
-                self._buf += ''.join(self._bufList)
+                self._buf += b''.join(self._bufList)
                 self._bufList = []
             # Find newline.
-            i = self._buf.find('\n', self._pos)
+            i = self._buf.find(b'\n', self._pos)
             if i < 0:
                 # Not found?
                 if self._eof:
@@ -218,12 +224,14 @@ class InputStream(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         r = self.readline()
         if not r:
             raise StopIteration
         return r
 
+    next = __next__
+
     def add_data(self, data):
         if not data:
             self._eof = True
@@ -323,7 +331,7 @@ class OutputStream(object):
     def flush(self):
         # Only need to flush if this OutputStream is actually buffered.
         if self._buffered:
-            data = ''.join(self._bufList)
+            data = b''.join(self._bufList)
             self._bufList = []
             self._write(data)
 
@@ -383,14 +391,14 @@ def decode_pair(s, pos=0):
     The number of bytes decoded as well as the name/value pair
     are returned.
     """
-    nameLength = ord(s[pos])
+    nameLength = struct.unpack('b', s[pos:pos+1])[0]
     if nameLength & 128:
         nameLength = struct.unpack('!L', s[pos:pos+4])[0] & 0x7fffffff
         pos += 4
     else:
         pos += 1
 
-    valueLength = ord(s[pos])
+    valueLength = struct.unpack('b', s[pos:pos+1])[0]
     if valueLength & 128:
         valueLength = struct.unpack('!L', s[pos:pos+4])[0] & 0x7fffffff
         pos += 4
@@ -412,15 +420,15 @@ def encode_pair(name, value):
     """
     nameLength = len(name)
     if nameLength < 128:
-        s = chr(nameLength)
+        s = struct.pack('b', nameLength)
     else:
-        s = struct.pack('!L', nameLength | 0x80000000L)
+        s = struct.pack('!L', nameLength | 0x80000000)
 
     valueLength = len(value)
     if valueLength < 128:
-        s += chr(valueLength)
+        s += struct.pack('b', valueLength)
     else:
-        s += struct.pack('!L', valueLength | 0x80000000L)
+        s += struct.pack('!L', valueLength | 0x80000000)
 
     return s + name + value
     
@@ -436,7 +444,7 @@ class Record(object):
         self.requestId = requestId
         self.contentLength = 0
         self.paddingLength = 0
-        self.contentData = ''
+        self.contentData = b''
 
     def _recvall(sock, length):
         """
@@ -448,8 +456,8 @@ class Record(object):
         while length:
             try:
                 data = sock.recv(length)
-            except socket.error, e:
-                if e[0] == errno.EAGAIN:
+            except socket.error as e:
+                if e.errno == errno.EAGAIN:
                     select.select([sock], [], [])
                     continue
                 else:
@@ -460,7 +468,7 @@ class Record(object):
             dataLen = len(data)
             recvLen += dataLen
             length -= dataLen
-        return ''.join(dataList), recvLen
+        return b''.join(dataList), recvLen
     _recvall = staticmethod(_recvall)
 
     def read(self, sock):
@@ -505,8 +513,8 @@ class Record(object):
         while length:
             try:
                 sent = sock.send(data)
-            except socket.error, e:
-                if e[0] == errno.EAGAIN:
+            except socket.error as e:
+                if e.errno == errno.EAGAIN:
                     select.select([], [sock], [])
                     continue
                 else:
@@ -531,7 +539,7 @@ class Record(object):
         if self.contentLength:
             self._sendall(sock, self.contentData)
         if self.paddingLength:
-            self._sendall(sock, '\x00'*self.paddingLength)
+            self._sendall(sock, b'\x00'*self.paddingLength)
             
 class Request(object):
     """
@@ -570,11 +578,11 @@ class Request(object):
         try:
             self._flush()
             self._end(appStatus, protocolStatus)
-        except socket.error, e:
-            if e[0] != errno.EPIPE:
+        except socket.error as e:
+            if e.errno != errno.EPIPE:
                 raise
 
-    def _end(self, appStatus=0L, protocolStatus=FCGI_REQUEST_COMPLETE):
+    def _end(self, appStatus=0, protocolStatus=FCGI_REQUEST_COMPLETE):
         self._conn.end_request(self, appStatus, protocolStatus)
         
     def _flush(self):
@@ -593,11 +601,11 @@ class CGIRequest(Request):
         self.server = server
         self.params = dict(os.environ)
         self.stdin = sys.stdin
-        self.stdout = StdoutWrapper(sys.stdout) # Oh, the humanity!
+        self.stdout = StdoutWrapper(sys.stdout.buffer) # Oh, the humanity!
         self.stderr = sys.stderr
-        self.data = StringIO.StringIO()
+        self.data = BytesIO()
         
-    def _end(self, appStatus=0L, protocolStatus=FCGI_REQUEST_COMPLETE):
+    def _end(self, appStatus=0, protocolStatus=FCGI_REQUEST_COMPLETE):
         sys.exit(appStatus)
 
     def _flush(self):
@@ -646,8 +654,8 @@ class Connection(object):
                 self.process_input()
             except (EOFError, KeyboardInterrupt):
                 break
-            except (select.error, socket.error), e:
-                if e[0] == errno.EBADF: # Socket was closed by Request.
+            except (select.error, socket.error) as e:
+                if e.errno == errno.EBADF: # Socket was closed by Request.
                     break
                 raise
 
@@ -696,7 +704,7 @@ class Connection(object):
         """
         rec.write(self._sock)
 
-    def end_request(self, req, appStatus=0L,
+    def end_request(self, req, appStatus=0,
                     protocolStatus=FCGI_REQUEST_COMPLETE, remove=True):
         """
         End a Request.
@@ -745,7 +753,7 @@ class Connection(object):
 
         if not self._multiplexed and self._requests:
             # Can't multiplex requests.
-            self.end_request(req, 0L, FCGI_CANT_MPX_CONN, remove=False)
+            self.end_request(req, 0, FCGI_CANT_MPX_CONN, remove=False)
         else:
             self._requests[inrec.requestId] = req
 
@@ -776,7 +784,7 @@ class Connection(object):
                 pos = 0
                 while pos < inrec.contentLength:
                     pos, (name, value) = decode_pair(inrec.contentData, pos)
-                    req.params[name] = value
+                    req.params[force_str(name)] = force_str(value)
             else:
                 self._start_request(req)
 
@@ -836,7 +844,7 @@ class MultiplexedConnection(Connection):
         finally:
             self._lock.release()
 
-    def end_request(self, req, appStatus=0L,
+    def end_request(self, req, appStatus=0,
                     protocolStatus=FCGI_REQUEST_COMPLETE, remove=True):
         self._lock.acquire()
         try:
@@ -979,11 +987,11 @@ class BaseFCGIServer(object):
                                  socket.SOCK_STREAM)
             try:
                 sock.getpeername()
-            except socket.error, e:
-                if e[0] == errno.ENOTSOCK:
+            except socket.error as e:
+                if e.errno == errno.ENOTSOCK:
                     # Not a socket, assume CGI context.
                     isFCGI = False
-                elif e[0] != errno.ENOTCONN:
+                elif e.errno != errno.ENOTCONN:
                     raise
 
             # FastCGI/CGI discrimination is broken on Mac OS X.
@@ -1061,7 +1069,7 @@ class BaseFCGIServer(object):
         result = None
 
         def write(data):
-            assert type(data) is str, 'write() argument must be string'
+            assert type(data) is bytes, 'write() argument must be bytes'
             assert headers_set, 'write() before start_response()'
 
             if not headers_sent:
@@ -1078,10 +1086,10 @@ class BaseFCGIServer(object):
                                                     str(len(data))))
                     except:
                         pass
-                s = 'Status: %s\r\n' % status
+                s = b'Status: ' + status + b'\r\n'
                 for header in responseHeaders:
-                    s += '%s: %s\r\n' % header
-                s += '\r\n'
+                    s += header[0].encode('ascii') + b': ' + header[1].encode('ascii') + b'\r\n'
+                s += b'\r\n'
                 req.stdout.write(s)
 
             req.stdout.write(data)
@@ -1092,16 +1100,18 @@ class BaseFCGIServer(object):
                 try:
                     if headers_sent:
                         # Re-raise if too late
-                        raise exc_info[0], exc_info[1], exc_info[2]
+                        raise exc_info[0](exc_info[1]).with_traceback(exc_info[2])
                 finally:
                     exc_info = None # avoid dangling circular ref
             else:
                 assert not headers_set, 'Headers already set!'
 
-            assert type(status) is str, 'Status must be a string'
+            assert isinstance(status, (bytes, text_type)), 'Status must be text or bytes'
+            if isinstance(status, text_type):
+                status = status.encode('latin-1')
             assert len(status) >= 4, 'Status must be at least 4 characters'
             assert int(status[:3]), 'Status must begin with 3-digit code'
-            assert status[3] == ' ', 'Status must have a space after code'
+            #assert status[3] == b' ', 'Status must have a space after code'
             assert type(response_headers) is list, 'Headers must be a list'
             if __debug__:
                 for name,val in response_headers:
@@ -1125,8 +1135,8 @@ class BaseFCGIServer(object):
                 finally:
                     if hasattr(result, 'close'):
                         result.close()
-            except socket.error, e:
-                if e[0] != errno.EPIPE:
+            except socket.error as e:
+                if e.errno != errno.EPIPE:
                     raise # Don't let EPIPE propagate beyond server
         finally:
             if not self.multithreaded:
@@ -1136,19 +1146,19 @@ class BaseFCGIServer(object):
 
     def _sanitizeEnv(self, environ):
         """Ensure certain values are present, if required by WSGI."""
-        if not environ.has_key('SCRIPT_NAME'):
+        if 'SCRIPT_NAME' not in environ:
             environ['SCRIPT_NAME'] = ''
 
         reqUri = None
-        if environ.has_key('REQUEST_URI'):
+        if 'REQUEST_URI' in environ:
             reqUri = environ['REQUEST_URI'].split('?', 1)
 
-        if not environ.has_key('PATH_INFO') or not environ['PATH_INFO']:
+        if 'PATH_INFO' not in environ or not environ['PATH_INFO']:
             if reqUri is not None:
                 environ['PATH_INFO'] = reqUri[0]
             else:
                 environ['PATH_INFO'] = ''
-        if not environ.has_key('QUERY_STRING') or not environ['QUERY_STRING']:
+        if 'QUERY_STRING' not in environ or not environ['QUERY_STRING']:
             if reqUri is not None and len(reqUri) > 1:
                 environ['QUERY_STRING'] = reqUri[1]
             else:
@@ -1160,7 +1170,7 @@ class BaseFCGIServer(object):
                              ('SERVER_NAME', 'localhost'),
                              ('SERVER_PORT', '80'),
                              ('SERVER_PROTOCOL', 'HTTP/1.0')]:
-            if not environ.has_key(name):
+            if name not in environ:
                 environ['wsgi.errors'].write('%s: missing FastCGI param %s '
                                              'required by WSGI!\n' %
                                              (self.__class__.__name__, name))
@@ -1173,10 +1183,10 @@ class BaseFCGIServer(object):
         """
         if self.debug:
             import cgitb
-            req.stdout.write('Content-Type: text/html\r\n\r\n' +
-                             cgitb.html(sys.exc_info()))
+            req.stdout.write(b'Content-Type: text/html\r\n\r\n' +
+                             cgitb.html(sys.exc_info()).encode('utf8'))
         else:
-            errorpage = """<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
+            errorpage = b"""<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
 <html><head>
 <title>Unhandled Exception</title>
 </head><body>
@@ -1184,5 +1194,5 @@ class BaseFCGIServer(object):
 <p>An unhandled exception was thrown by the application.</p>
 </body></html>
 """
-            req.stdout.write('Content-Type: text/html\r\n\r\n' +
+            req.stdout.write(b'Content-Type: text/html\r\n\r\n' +
                              errorpage)
