@@ -142,7 +142,7 @@ class WSGIServer(BaseSCGIServer, PreforkServer):
 
         try:
             sock = self._setupSocket()
-        except socket.error, e:
+        except socket.error as e:
             self.logger.error('Failed to bind socket (%s), exiting', e[1])
             return False
 
@@ -155,10 +155,6 @@ class WSGIServer(BaseSCGIServer, PreforkServer):
 
         return ret
 
-def factory(global_conf, host=None, port=None, **local):
-    import paste_factory
-    return paste_factory.helper(WSGIServer, global_conf, host, port, **local)
-
 if __name__ == '__main__':
     def test_app(environ, start_response):
         """Probably not the most efficient example."""
@@ -172,7 +168,7 @@ if __name__ == '__main__':
         names.sort()
         for name in names:
             yield '<tr><td>%s</td><td>%s</td></tr>\n' % (
-                name, cgi.escape(`environ[name]`))
+                name, cgi.escape(repr(environ[name])))
 
         form = cgi.FieldStorage(fp=environ['wsgi.input'], environ=environ,
                                 keep_blank_values=1)
